<?php
/**
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author zahls.ch <info@zahls.ch>
 * @copyright  siebenberge gmbh
 * @license MIT License
 */

class ZahlsGatewayModuleFrontController extends ModuleFrontController
{

    public function initContent()
    {
        $transaction = Tools::getValue('transaction');
        $id_cart = $transaction['invoice']['referenceId'];

        if (empty($id_cart) || !isset($transaction['status'])) {
            die;
        }

        $gateway = $this->getZahlsGateway((int)$id_cart);
        if (!$gateway) {
            PrestaShopLoggerCore::addLog('GATEWAY FOR CART ID: ' . $id_cart . ' NOT FOUND');
        }

        $status = $gateway->getStatus();
        if (empty($status) || $status !== $transaction['status'] || !in_array($status, ['confirmed', 'waiting'])) {
            die;
        }

        $zahlsModule = Module::getInstanceByName('zahls');
        $cart = new Cart((int)$id_cart);
        if (Validate::isLoadedObject($cart) && $cart->OrderExists()) {
            die();
        }

        $customer = new Customer($cart->id_customer);
        try {
            $zahlsModule->validateOrder(
                (int)$id_cart,
                (int)Configuration::get('PS_OS_PAYMENT'),
                (float)$gateway->getAmount() / 100,
                'Zahls',
                null,
                array(),
                (int)$cart->id_currency,
                false,
                $customer->secure_key
            );
        } catch (PrestaShopException $e) {
            PrestaShopLoggerCore::addLog('CART ID: ' . $id_cart . ' - ' . $e->getMessage());
        }
        die();
    }

    /**
     * Get Zahls Gateway from the cart id
     *
     * @param int $id_cart Cart id
     * @return \Zahls\Models\Request\Gateway|NULL
     */
    public function getZahlsGateway($id_cart)
    {
        $id_gateway = static::getCartGatewayId($id_cart);

        if (!$id_gateway) {
            return;
        }
        spl_autoload_register(function ($class) {
            $root = _PS_MODULE_DIR_ . '/zahls/controllers/front/zahls-ch-php';
            $classFile = $root . '/lib/' . str_replace('\\', '/', $class) . '.php';
            if (file_exists($classFile)) {
                require_once $classFile;
            }
        });

        $instanceName = Configuration::get('ZAHLS_INSTANCE_NAME');
        $secret = Configuration::get('ZAHLS_API_SECRET');
        $zahls = new \Zahls\Zahls($instanceName, $secret);
        $gateway = new \Zahls\Models\Request\Gateway();
        $gateway->setId($id_gateway);

        try {
            return $zahls->getOne($gateway);
        } catch (\Zahls\ZahlsException $e) {
            return;
        }
    }

    /**
     * Get Gateway id from the cart id
     *
     * @param int $id_cart cart id
     * @return int
     */
    public static function getCartGatewayId($id_cart)
    {
        if (empty($id_cart)) {
            return 0;
        }

        return (int)Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue('
            SELECT id_gateway FROM `' . _DB_PREFIX_ . 'zahls_gateway`
            WHERE id_cart = ' . (int)$id_cart);
    }
}
