<?php
/**
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author zahls.ch <info@zahls.ch>
 * @copyright  siebenberge gmbh
 * @license MIT License
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

class Zahls extends PaymentModule
{
    public function __construct()
    {
        $this->name = 'zahls';
        $this->tab = 'payments_gateways';
        $this->module_key = 'ac51256c4accd367ac905116fb1e0e0a';
        $this->version = '1.0.0';
        $this->author = 'siebenberge gmbh';
        $this->is_eu_compatible = 1;
        $this->ps_versions_compliancy = array('min' => '1.7');
        $this->controllers = array('payment', 'validation', 'gateway');
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('zahls.ch credit cards and TWINT');
        $this->description = $this->l('Accept payments with credit cards and TWINT');
        $this->confirmUninstall = $this->l('Are you sure you want to uninstall?');
        $this->validateDb();
    }

    private function validateDb()
    {
        Db::getInstance()->execute('
            CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'zahls_gateway` (
                id_cart INT(11) NOT NULL UNIQUE,
                id_gateway INT(11) UNSIGNED DEFAULT "0" NOT NULL,
                PRIMARY KEY (`id_cart`)
            ) DEFAULT CHARSET=utf8');
    }

    public function install()
    {
        // Install default
        if (!parent::install() || !$this->installDb() || !$this->registrationHook()) {
            return false;
        }

        if (!Configuration::updateValue('ZAHLS_LABEL', '')
            || !Configuration::updateValue('ZAHLS_DESCRIPTION', '')
            || !Configuration::updateValue('ZAHLS_API_SECRET', '')
            || !Configuration::updateValue('ZAHLS_INSTANCE_NAME', '')
            || !Configuration::updateValue('ZAHLS_USE_MODAL', '')
            || !Configuration::updateValue('ZAHLS_PAY_ICONS', '')
        ) {
            return false;
        }

        return true;
    }

    /**
     * Install DataBase table
     * @return boolean if install was successfull
     */
    private function installDb()
    {
        return Db::getInstance()->execute('
            CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'zahls_gateway` (
                id_cart INT(11) NOT NULL UNIQUE,
                id_gateway INT(11) UNSIGNED DEFAULT "0" NOT NULL,
                PRIMARY KEY (`id_cart`)
            ) DEFAULT CHARSET=utf8');
    }

    /**
     * [registrationHook description]
     * @return [type] [description]
     */
    private function registrationHook()
    {
        if (_PS_VERSION_ >= '1.7' && !$this->registerHook('paymentOptions')) {
            return false;
        } elseif (_PS_VERSION_ < '1.7' &&
            (!$this->registerHook('payment') || !$this->registerHook('paymentReturn'))) {
            return false;
        }

        return true;
    }

    public function uninstall()
    {
        $config = array(
            'ZAHLS_LABEL',
            'ZAHLS_DESCRIPTION',
            'ZAHLS_API_SECRET',
            'ZAHLS_INSTANCE_NAME',
            'ZAHLS_USE_MODAL',
            'ZAHLS_PAY_ICONS',
        );
        foreach ($config as $var) {
            Configuration::deleteByName($var);
        }

        //Uninstall DataBase
        if (!$this->uninstallDb()) {
            return false;
        }

        // Uninstall default
        if (!parent::uninstall()) {
            return false;
        }
        return true;
    }

    /**
     * Uninstall DataBase table
     * @return boolean if install was successfull
     */
    private function uninstallDb()
    {
        return Db::getInstance()->execute('DROP TABLE `' . _DB_PREFIX_ . 'zahls_gateway`');
    }
    
    
    
    private function displayCheck()
    {
        
        $this->smarty->assign(['path' => $this->context->shop->domain]);
        return $this->display(__FILE__, './views/templates/hook/infos.tpl');
    }


    public function getContent()
    {
        $this->postProcess();
        $form = $this->displayCheck();

        $options = array(
            array('id_option' => 'twint', 'name' => 'TWINT'),
            array('id_option' => 'mastercard', 'name' => 'Mastercard',),
            array('id_option' => 'visa', 'name' => 'Visa',),
            array('id_option' => 'apple_pay', 'name' => 'Apple Pay',),
            array('id_option' => 'google_pay', 'name' => 'Google Pay'),
            array('id_option' => 'masterpass', 'name' => 'Masterpass',),
            array('id_option' => 'american_express', 'name' => 'American Express',),
            array('id_option' => 'maestro', 'name' => 'Maestro',),
            array('id_option' => 'jcb', 'name' => 'JCB',),
            array('id_option' => 'wirpay', 'name' => 'WIRpay',),
            array('id_option' => 'paypal', 'name' => 'PayPal',),
            array('id_option' => 'bitcoin', 'name' => 'Bitcoin',),
            array('id_option' => 'sofortueberweisung_de', 'name' => 'Sofort Ueberweisung',),
            array('id_option' => 'airplus', 'name' => 'Airplus',),
            array('id_option' => 'billpay', 'name' => 'Billpay',),
            array('id_option' => 'bonuscard', 'name' => 'Bonus card',),
            array('id_option' => 'cashu', 'name' => 'CashU',),
            array('id_option' => 'cb', 'name' => 'Carte Bleue',),
            array('id_option' => 'diners_club', 'name' => 'Diners Club',),
            array('id_option' => 'direct_debit', 'name' => 'Direct Debit',),
            array('id_option' => 'discover', 'name' => 'Discover',),
            array('id_option' => 'elv', 'name' => 'ELV',),
            array('id_option' => 'ideal', 'name' => 'iDEAL',),
            array('id_option' => 'invoice', 'name' => 'Invoice',),
            array('id_option' => 'myone', 'name' => 'My One',),
            array('id_option' => 'paysafecard', 'name' => 'Paysafe Card',),
            array('id_option' => 'postfinance_card', 'name' => 'PostFinance Card',),
            array('id_option' => 'postfinance_efinance', 'name' => 'PostFinance E-Finance',),
            array('id_option' => 'swissbilling', 'name' => 'SwissBilling',),
            array('id_option' => 'barzahlen', 'name' => 'Barzahlen/Viacash'),
            array('id_option' => 'bancontact', 'name' => 'Bancontact'),
            array('id_option' => 'giropay', 'name' => 'GiroPay'),
            array('id_option' => 'eps', 'name' => 'EPS'),
            
        );

        $fields_form = array();
        $fields_form[0]['form'] = array(
            'legend' => array(
                'title' => $this->l('Settings'),
            ),
            'input' => array(
                array(
                    'type' => 'text',
                    'label' => $this->l('Title'),
                    'name' => 'zahls_label',
                    'required' => true
                ),
                array(
                    'type' => 'textarea',
                    'label' => $this->l('Description'),
                    'name' => 'zahls_description',
                    'required' => true
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('API Secret'),
                    'name' => 'zahls_api_secret',
                    'required' => true
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('Instance'),
                    'name' => 'zahls_instance_name',
                    'required' => true
                ),

                array(
                    'type' => 'select',
                    'label' => $this->l('Payment Icons'),
                    'name' => 'zahls_pay_icons',
                    'multiple' => true,
                    'options' => array(
                        'query' => $options,
                        'id' => 'id_option',
                        'name' => 'name',
                    )
                ),
            ),
            'submit' => array(
                'title' => $this->l('Save'),
                'class' => 'btn btn-default pull-right'
            )
        );

        $fields_value = array(
            'zahls_label' => Configuration::get('ZAHLS_LABEL'),
            'zahls_description' => Configuration::get('ZAHLS_DESCRIPTION'),
            'zahls_api_secret' => Configuration::get('ZAHLS_API_SECRET'),
            'zahls_instance_name' => Configuration::get('ZAHLS_INSTANCE_NAME'),
            'zahls_use_modal' => (bool)Configuration::get('ZAHLS_USE_MODAL'),
            'zahls_pay_icons[]' => unserialize(Configuration::get('ZAHLS_PAY_ICONS')),
        );
        $helper = new HelperForm();
        $helper->module = $this;
        $helper->name_controller = $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex . '&configure=' . $this->name;
        $helper->title = $this->displayName;
        $helper->show_toolbar = true;
        $helper->submit_action = 'zahls_config';
        $default_lang = (int)Configuration::get('PS_LANG_DEFAULT');
        $helper->default_form_language = $default_lang;
        $helper->allow_employee_form_lang = $default_lang;
        $helper->tpl_vars = array(
            'fields_value' => $fields_value,
            'id_language' => $this->context->language->id,
            'back_url' => $this->context->link->getAdminLink('AdminModules')
                . '&configure=' . $this->name
                . '&tab_module=' . $this->tab
                . '&module_name=' . $this->name
                . '#paypal_params'
        );
        
    
        
        
        $form .= $helper->generateForm($fields_form);

        return $form;
    }
    
    
    public function renderForm()
    {
        $fields_form = [
            'form' => [
                'legend' => [
                    'title' => $this->trans('Contact details', [], 'Modules.Checkpayment.Admin'),
                    'icon' => 'icon-envelope',
                ],
                'input' => [
                    [
                        'type' => 'text',
                        'label' => $this->trans('Payee (name)', [], 'Modules.Checkpayment.Admin'),
                        'name' => 'CHEQUE_NAME',
                        'required' => true,
                    ],
                    [
                        'type' => 'textarea',
                        'label' => $this->trans('Address', [], 'Modules.Checkpayment.Admin'),
                        'desc' => $this->trans(
                            'Address where the check should be sent to.',
                            [],
                            'Modules.Checkpayment.Admin'
                        ),
                        'name' => 'CHEQUE_ADDRESS',
                        'required' => true,
                    ],
                ],
                'submit' => [
                    'title' => $this->trans('Save', [], 'Admin.Actions'),
                ],
            ],
        ];

        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->id = (int) Tools::getValue('id_carrier');
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'btnSubmit';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false) .
            '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = [
            'fields_value' => $this->getConfigFieldsValues(),
        ];

        return $helper->generateForm([$fields_form]);
    }
    
    
    
    

    private function postProcess()
    {
        if (Tools::isSubmit('zahls_config')) {
            $instance = Tools::getValue('zahls_instance_name');
            $instance = str_replace(".zahls.ch", "", $instance);
            $instance = str_replace(".ch", "", $instance);
            
            Configuration::updateValue('ZAHLS_LABEL', Tools::getValue('zahls_label'));
            Configuration::updateValue('ZAHLS_DESCRIPTION', Tools::getValue('zahls_description'));
            Configuration::updateValue('ZAHLS_API_SECRET', Tools::getValue('zahls_api_secret'));
            Configuration::updateValue('ZAHLS_INSTANCE_NAME', $instance);
            Configuration::updateValue('ZAHLS_USE_MODAL', Tools::getValue('zahls_use_modal'));
            Configuration::updateValue('ZAHLS_PAY_ICONS', serialize(Tools::getValue('zahls_pay_icons')));
        }
    }


    // Payment hook for version >= 1.7
    public function hookPaymentOptions($params)
    {
        $payIconSource = unserialize(Configuration::get('ZAHLS_PAY_ICONS'));

        $payment_option = new PrestaShop\PrestaShop\Core\Payment\PaymentOption();
        $action_text = $this->l(Configuration::get('ZAHLS_LABEL'));
        $this->context->smarty->assign(array(
            'path' => $this->_path,
        ));
        $payment_option->setCallToActionText($action_text);
        $payment_option->setAction($this->context->link->getModuleLink($this->name, 'zahls'));
        
        $this->smarty->assign([
            'description' => Configuration::get('ZAHLS_DESCRIPTION'),
            'icons' => $payIconSource,
            'path' => $this->_path,
        ]);
        $paymentForm = $this->fetch('module:zahls/views/templates/hook/payment_options.tpl');
        $payment_option->setAdditionalInformation($paymentForm);


        $payment_options = array(
            $payment_option,
        );

        return $payment_options;
    }
}
